#!/usr/bin/perl

use File::Path;
use File::Copy;
use File::Glob ":glob";
use POSIX ":sys_wait_h";

$timeout = 60;

%{ENV}->{"MOZILLA_FIVE_HOME"}="/usr/lib/firefox-1.0.7";
%{ENV}->{"LD_LIBRARY_PATH"}="/usr/lib/firefox-1.0.7";

umask 022;

if ( -f "$MOZ_DIST_BIN/regxpcom" )
{
    # remove all of the old files
    rmtree("$MOZ_DIST_BIN/chrome/overlayinfo");
    unlink <$MOZ_DIST_BIN/chrome/*.rdf>;
    unlink("$MOZ_DIST_BIN/component.reg");
    unlink("$MOZ_DIST_BIN/components/compreg.dat");
    unlink("$MOZ_DIST_BIN/components/xpti.dat");

    # create a new clean path
    mkpath("$MOZ_DIST_BIN/chrome/overlayinfo");

    # rebuild the installed-chrome.txt file from the installed
    # languages
    if ( -f "$MOZ_DIST_BIN/chrome/lang/installed-chrome.txt" ) {
	rebuild_lang_files();
    }

    # run regxpcom
    $pid = fork();

    # I am the child.
    if ($pid == 0) {
	exec("$MOZ_DIST_BIN/regxpcom > /dev/null 2> /dev/null");
    }
    # I am the parent.
    else {
	my $timepassed = 0;
	do {
	    $kid = waitpid($pid, &WNOHANG);
	    sleep(1);
	    $timepassed++;
        } until $kid == -1 || $timepassed > $timeout;

	# should we kill?
	if ($timepassed > $timeout) {
	    kill (9, $pid);
	    # kill -9 can leave threads hanging around
	    system("/usr/bin/killall -9 regxpcom");
	}
    }

    # and run regchrome for good measure
    $pid = fork();

    # I am the child.
    if ($pid == 0) {
	exec("$MOZ_DIST_BIN/regchrome > /dev/null 2> /dev/null");
    }
    # I am the parent.
    else {
	my $timepassed = 0;
	do {
	    $kid = waitpid($pid, &WNOHANG);
	    sleep(1);
	    $timepassed++;
        } until $kid == -1 || $timepassed > $timeout;

	# should we kill?
	if ($timepassed > $timeout) {
	    kill (9, $pid);
	    # kill -9 can leave threads hanging around
	    system("/usr/bin/killall -9 regchrome");
	}
    }

}


sub rebuild_lang_files {
    unlink("$MOZ_DIST_BIN/chrome/installed-chrome.txt");

    open (OUTPUT, "+>", "$MOZ_DIST_BIN/chrome/installed-chrome.txt")||
	die("Failed to open installed-chrome.txt: $!\n");

    copy("$MOZ_DIST_BIN/chrome/lang/installed-chrome.txt",
	 \*OUTPUT);

    foreach (bsd_glob("$MOZ_DIST_BIN/chrome/lang/lang-*.txt")) {
	copy($_, \*OUTPUT);
    }

    copy("$MOZ_DIST_BIN/chrome/lang/default.txt",
	 \*OUTPUT);
}
